{"cells":[{"cell_type":"markdown","metadata":{},"source":["# HilosJavaBasico"]},{"cell_type":"code","execution_count":null,"metadata":{"vscode":{"languageId":"java"}},"outputs":[],"source":["System.out.println(\"Hola Mundo desde Notebook\")"]},{"cell_type":"markdown","metadata":{"id":"XgEQSLmLCxCz"},"source":["## Hilos en Java\n","Los hilos son una característica importante en la programación concurrente, y permiten que un programa ejecute múltiples tareas al mismo tiempo. En Java, esto se logra mediante la clase *'Thread'* y la interfaz *'Runnable'*."]},{"cell_type":"markdown","metadata":{"id":"V01Albk9CxC2"},"source":["### 1. La clase Thread:\n","\n","Java proporciona una clase incorporada llamada Thread que se encuentra en el paquete java.lang. Puedes crear un hilo extendiendo esta clase e implementando el método run()."]},{"cell_type":"code","execution_count":null,"metadata":{"id":"oyRg4tvcCxC3","vscode":{"languageId":"java"}},"outputs":[],"source":["\n","class MiHilo extends Thread {\n","    public void run() { \n","        try { // se debe implementar esto ya que no se permiten colocar sleeps si no se incrustan dentro de excepciones.\n","            System.out.println(\"Hola Mundo desde un Hilo heredado\");\n","            Thread.sleep(2000);\n","        } catch (Exception e) { // Si se da una excepción de algun tipo, se imprime el mensaje de interrupción.\n","            System.out.println(\"Ejecución interrumpida\");\n","        }\n","    }\n","}"]},{"cell_type":"markdown","metadata":{"id":"A9TLXOa5CxC4"},"source":["#### Iniciar un hilo\n","Para inciar un hilo primero debes instanciar la clase MiHilo.\n","\n","Para ejecutar el hilo debes llamar el método Start directamente en tu objeto. Este ejecutará el procedimiento run."]},{"cell_type":"code","execution_count":null,"metadata":{"id":"uMBlWFz_CxC5","vscode":{"languageId":"java"}},"outputs":[],"source":["public class Clase {\n","    public static void main() {\n","        // Crear una instancia de MiHilo\n","        MiHilo miHilo = new MiHilo();\n","\n","        System.out.println(\"Hola Mundo desde El main\");\n","        // Si extiende la clase Thread\n","        miHilo.start();\n","\n","        System.out.println(\"Fin de la ejecución\");\n","    }\n","}"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"tWDK7UknCxC5","outputId":"b450137f-7fc2-4110-af72-f2455b8f6ca6","vscode":{"languageId":"java"}},"outputs":[{"name":"stdout","output_type":"stream","text":["Hola Mundo desde El main\n","Fin de la ejecución\n","Hola Mundo desde un Hilo heredado\n"]}],"source":["Clase.main(); "]},{"cell_type":"markdown","metadata":{"id":"kYDKtWJMCxC6"},"source":["### 2. La interfaz Runnable:\n","\n","Otra forma de crear un hilo es implementar la interfaz *Runnable*. Esta interfaz solo tiene un método llamado *run()*, que debes implementar en tu clase."]},{"cell_type":"code","execution_count":null,"metadata":{"id":"UabLLEoeCxC7","vscode":{"languageId":"java"}},"outputs":[],"source":["class MiHiloR implements Runnable {\n","    public void run(){\n","        \n","\n","        try { // se debe implementar esto ya que no se permiten colocar sleeps si no se incrustan dentro de excepciones.\n","            System.out.println(\"Hola Mundo desde un Hilo desde interfaz\");\n","            Thread.sleep(2000);\n","        } catch (Exception e) { // Si se da una excepción de algun tipo, se imprime el mensaje de interrupción.\n","            System.out.println(\"Ejecución interrumpida\");\n","        }\n","\n","    }\n","}"]},{"cell_type":"markdown","metadata":{"id":"8IoZMCpTCxC7"},"source":["#### Iniciar un hilo\n","Para inciar un hilo primero debes instanciar la clase MiHiloR de tipo Thread.\n","\n","Para ejecutar el hilo debes llamar el método Start directamente en tu objeto. Este ejecutará el procedimiento run."]},{"cell_type":"code","execution_count":null,"metadata":{"id":"cWXSj1IuCxC8","vscode":{"languageId":"java"}},"outputs":[],"source":["public class Clase2{\n","    public static void main() {\n","        System.out.println(\"Inicio Programa Main\");\n","        MiHiloR Runnable1 = new MiHiloR();\n","\n","        Thread hilo1 = new Thread(Runnable1);\n","\n","        hilo1.start();\n","\n","        System.out.println(\"Final del programa Main\");\n","    }\n","}"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"46orrgrzCxC8","outputId":"6da405e9-a9f1-40e5-ce7d-e70154111f44","vscode":{"languageId":"java"}},"outputs":[{"name":"stdout","output_type":"stream","text":["Inicio Programa Main\n","Final del programa Main\n","Hola Mundo desde un Hilo desde interfaz"]}],"source":["Clase2.main(); // Tuvimos que parar de golpe la ejecución."]},{"cell_type":"markdown","metadata":{"id":"aGXggv9hCxC9"},"source":["### Diferencias entre Threads y Runnable \n","\n","A continuación, se presentan las diferencias clave entre ambas:\n","\n","1. **Herencia**: Cuando extiendes la clase Thread, estás utilizando la herencia para crear un hilo. Esto significa que tu clase heredará todas las propiedades y métodos de la clase Thread. Sin embargo, Java no admite la herencia múltiple, lo que significa que si tu clase ya hereda de otra clase, no puedes extender Thread al mismo tiempo. En cambio, cuando implementas la interfaz Runnable, puedes heredar de otra clase y también crear un hilo al mismo tiempo, ya que Java admite la implementación de múltiples interfaces.\n","\n","2. **Reutilización del código** : La implementación de la interfaz Runnable es más flexible en términos de reutilización de código. Si extiendes la clase Thread, no puedes separar el código del hilo de la propia clase Thread. Por otro lado, cuando implementas la interfaz Runnable, puedes reutilizar la misma instancia de la clase que implementa Runnable en varios hilos.\n","\n","3. **Sobrecarga**: Al extender la clase Thread, cada hilo crea una nueva instancia de la clase y, por lo tanto, consume más memoria en comparación con la implementación de la interfaz Runnable. En el caso de Runnable, puedes crear múltiples hilos utilizando una única instancia de la clase que implementa Runnable, lo que resulta en una menor sobrecarga de memoria.\n","\n","_En general, se recomienda utilizar la interfaz Runnable en lugar de extender la clase Thread debido a las ventajas mencionadas. Sin embargo, en algunos casos específicos, como cuando es necesario sobrescribir otros métodos de la clase Thread, extender Thread puede ser la opción adecuada._"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"Cz7vxNyjCxC-","vscode":{"languageId":"java"}},"outputs":[],"source":["\n","// Este ejemplo implementa un hilo que cuando se llama, imprime el nombre del proceso seguido de un número que se incrementa\n","// Luego de imprimirlo, espera 1 segundo y luego vuelve\n","\n","class MyRunnable implements Runnable {\n","    private String name;\n","    private int wait_time;\n","\n","    public MyRunnable(String name, int wait_time) {\n","        this.name = name;\n","        this.wait_time = wait_time;\n","    }\n","\n","    @Override\n","    public void run() {\n","        System.out.println(\"Iniciando proceso\"+name);\n","        for (int i = 0; i < 5; i++) {\n","            System.out.println(name + \": \" + i);\n","            try {\n","                Thread.sleep(wait_time);\n","            } catch (Exception e) {\n","                System.out.println(name + \" ha sido interrumpido.\");\n","            }\n","        }\n","        System.out.println(name + \" ha terminado.\");\n","    }\n","}"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"VKD9p9rPCxC-","vscode":{"languageId":"java"}},"outputs":[],"source":["public class Clase3{\n","    public static void main() {\n","        System.out.println(\"Ingresando al Main\");\n","        MyRunnable runnable1 = new MyRunnable(\"Hilo 1\",1000);\n","\n","        MyRunnable runnable2 = new MyRunnable(\"Hilo 2\",2000);\n","\n","        Thread hilo1 = new Thread(runnable1);\n","        Thread hilo2 = new Thread(runnable2);\n","\n","        hilo1.start();\n","        hilo2.start();\n","\n","        try { // Por reglas de java el siguiente método se debe colocar en una exception para evitar\n","            hilo1.join(); // este método se utiliza para esperar que se terminen de ejecutar los procesos, \n","                          // de lo contrario se sigue ejecutando el main mientras se ejecutan los hilos.\n","            hilo2.join();\n","        } catch (Exception e) {\n","            System.out.println(\"___\");\n","        }\n","\n","        System.out.println(\"Terminando el proceso Main\");\n","    }\n","}\n"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"VH96BXS0CxC-","outputId":"f32de635-737e-4474-c9fd-e0e22cee1223","vscode":{"languageId":"java"}},"outputs":[{"name":"stdout","output_type":"stream","text":["Ingresando al Main\n","Iniciando procesoHilo 2\n","Iniciando procesoHilo 1\n","Hilo 2: 0\n","Hilo 1: 0\n","Hilo 1: 1\n","Hilo 2: 1\n","Hilo 1: 2\n","Hilo 1: 3\n","Hilo 2: 2\n","Hilo 1: 4\n","Hilo 1 ha terminado.\n","Hilo 2: 3\n","Hilo 2: 4\n","Hilo 2 ha terminado.\n","Terminando el proceso Main\n"]}],"source":["Clase3.main();"]},{"cell_type":"markdown","metadata":{"id":"-Aal7Qv1CxC_"},"source":["### Sincronización de Hilos\n","Uno de los problemas clave en la programación concurrente es garantizar que los hilos accedan a los recursos compartidos de manera segura y controlada. La sincronización en Java ayuda a prevenir condiciones de carrera y garantiza que solo un hilo pueda acceder a un recurso compartido a la vez. Puedes sincronizar bloques de código utilizando la palabra clave *synchronized*."]},{"cell_type":"code","execution_count":null,"metadata":{"id":"HUtcTJfrCxC_","vscode":{"languageId":"java"}},"outputs":[],"source":["public synchronized void metodoSincronizado() {\n","    // Código sincronizado\n","}"]},{"cell_type":"markdown","metadata":{"id":"HFjoDMgoCxDA"},"source":["#### Ejemplo"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"GV4KYA_7CxDA","vscode":{"languageId":"java"}},"outputs":[],"source":["// Este es el objeto contador al que vamos a acceder\n","class Contador {\n","    private int valor;\n","\n","    public Contador() {\n","        this.valor = 0;\n","    }\n","\n","    // Método sincronizado para incrementar el contador, con esto se evita que dos hilos lo accedan al mismo tiempo\n","    public synchronized void incrementar() {\n","        valor++;\n","    }\n","    // Método sincronizado para obtener el valor actual del contador\n","    public synchronized int getValor() {\n","        return valor;\n","    }\n","}"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"RFnCDAeMCxDA","vscode":{"languageId":"java"}},"outputs":[],"source":["class Incrementador implements Runnable {\n","    // dentro de la clase que implementa la interfaz vamos a declarar un atributo de tipo contador\n","    private Contador contador;\n","\n","    public Incrementador(Contador contador) { // En el constructor de la clase,  asignamos el objeto contador que debe ser enviado desde el main.\n","        this.contador = contador;\n","    }\n","\n","    @Override \n","    public void run() {\n","        for (int i = 0; i < 5; i++) {\n","            contador.incrementar();\n","            System.out.println(\"El hilo valor del contador es:\"+contador.getValor());\n","        }\n","    }\n","}"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"q_eT3cbkCxDB","vscode":{"languageId":"java"}},"outputs":[],"source":["public class Clase4 {\n","    public static void main() throws InterruptedException {\n","        System.out.println(\"Empezamos el programa main\");\n","        Contador contador = new Contador();\n","        Incrementador incrementador = new Incrementador(contador);\n","\n","        // Crear 5 hilos que incrementen el contador\n","        Thread[] hilos = new Thread[5];\n","        for (int i = 0; i < 5; i++) {\n","            hilos[i] = new Thread(incrementador);\n","            hilos[i].start();\n","        }\n","\n","        // Esperar a que todos los hilos terminen\n","        for (int i = 0; i < 5; i++) {\n","            hilos[i].join();\n","        }\n","\n","        System.out.println(\"Valor final del contador: \" + contador.getValor());\n","    }\n","}"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"Jc6mCFNoCxDB","outputId":"bc476fd5-8e93-4e4f-9571-de19fdc5a438","vscode":{"languageId":"java"}},"outputs":[{"name":"stdout","output_type":"stream","text":["Empezamos el programa main\n","El hilo valor del contador es:2\n","El hilo valor del contador es:5\n","El hilo valor del contador es:6\n","El hilo valor del contador es:8\n","El hilo valor del contador es:9\n","El hilo valor del contador es:10\n","El hilo valor del contador es:4\n","El hilo valor del contador es:3\n","El hilo valor del contador es:1\n","El hilo valor del contador es:13\n","El hilo valor del contador es:14\n","El hilo valor del contador es:15\n","El hilo valor del contador es:12\n","El hilo valor del contador es:11\n","El hilo valor del contador es:18\n","El hilo valor del contador es:7\n","El hilo valor del contador es:20\n","El hilo valor del contador es:19\n","El hilo valor del contador es:17\n","El hilo valor del contador es:16\n","El hilo valor del contador es:23\n","El hilo valor del contador es:24\n","El hilo valor del contador es:22\n","El hilo valor del contador es:21\n","El hilo valor del contador es:25\n","Valor final del contador: 25\n"]}],"source":["Clase4.main();"]},{"cell_type":"markdown","metadata":{"id":"GzVZoM_TCxDB"},"source":["En este ejemplo, hay un objeto Contador que mantiene un contador y proporciona métodos para incrementarlo y obtener su valor. La clase Incrementador implementa Runnable y, en su método run, incrementa el contador 5 veces. En el método _*main*_, creamos 5 hilos que incrementan el contador simultáneamente.\n","\n","Dado que el método *_incrementar()_* en la clase Contador es *_sincronizado_*, garantizamos que solo un hilo pueda acceder a él a la vez. Esto evita condiciones de carrera y asegura que el contador se incremente de manera correcta y segura. Si no utilizáramos la sincronización, podríamos obtener resultados incorrectos debido a las condiciones de carrera entre los hilos."]},{"cell_type":"markdown","metadata":{"id":"9LP1vTRMCxDC"},"source":["### Otros métodos de la clase Thread.\n","\n","* isInterrupted(): Comprueba si el hilo ha sido interrumpido. Este método devuelve _*true*_ si el hilo ha sido interrumpido y _*false*_ en caso contrario."]},{"cell_type":"code","execution_count":null,"metadata":{"id":"HUchk6PlCxDC","vscode":{"languageId":"java"}},"outputs":[],"source":["class MiHilo extends Thread {\n","    @Override\n","    public void run() {\n","        int i = 0;\n","        while (!isInterrupted()) {\n","            System.out.println(\"Ejecutando el hilo: \" + getName() + \", iteración: \" + i);\n","            i++;\n","            try {\n","                Thread.sleep(1000); // Hace que el hilo duerma durante 1 segundo\n","            } catch (Exception e) {\n","                interrupt(); // Establece el estado de interrupción del hilo\n","            }\n","        }\n","    }\n","}\n","\n","public class Clase6 {\n","    public static void main() {\n","        \n","        MiHilo hilo = new MiHilo();\n","        hilo.start(); // Inicia la ejecución del hilo\n","\n","        try {\n","            Thread.sleep(4000); // Hace que el hilo principal duerma durante 4 segundos\n","        } catch (Exception e) {\n","            e.printStackTrace();\n","        }\n","\n","        hilo.interrupt(); // Interrumpe el hilo en ejecución\n","        System.out.println(\"Terminando la ejecución del hilo\");\n","    }\n","}"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"L4_0DVg_CxDD","outputId":"61e72cc2-b034-45aa-fd12-aec6542ae89b","vscode":{"languageId":"java"}},"outputs":[{"name":"stdout","output_type":"stream","text":["Ejecutando el hilo: Thread-1, iteración: 0\n","Ejecutando el hilo: Thread-1, iteración: 1\n","Ejecutando el hilo: Thread-1, iteración: 2\n","Ejecutando el hilo: Thread-1, iteración: 3\n","Terminando la ejecución del hilo\n"]}],"source":["Clase6.main();"]},{"cell_type":"markdown","metadata":{"id":"cUEotmkKCxDD"},"source":["* **setName(String name)** y **getName()**: Establece y obtiene el nombre del hilo, respectivamente."]},{"cell_type":"code","execution_count":null,"metadata":{"id":"GzwzIA7cCxDD","vscode":{"languageId":"java"}},"outputs":[],"source":["class MiHilo extends Thread {\n","    public MiHilo(String name) {\n","        setName(name);\n","    }\n","\n","    @Override\n","    public void run() {\n","        System.out.println(\"Ejecutando el hilo: \" + getName());\n","    }\n","}\n","\n","public class Clase7 {\n","    public static void main() {\n","        MiHilo hilo1 = new MiHilo(\"Hilo 1\");\n","        MiHilo hilo2 = new MiHilo(\"Hilo 2\");\n","\n","        hilo1.start(); // Inicia la ejecución del hilo1\n","        hilo2.start(); // Inicia la ejecución del hilo2\n","\n","        \n","        try {\n","            hilo1.join();\n","            hilo2.join();\n","        } catch (Exception e) {\n","            System.out.println(\"__\");\n","        }\n","        \n","    }\n","}"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"nDDngJAvCxDE","outputId":"5aae9ea9-948a-412f-a92b-a35dceb04af2","vscode":{"languageId":"java"}},"outputs":[{"name":"stdout","output_type":"stream","text":["Ejecutando el hilo: Hilo 2\n","Ejecutando el hilo: Hilo 1\n"]}],"source":["Clase7.main();"]},{"cell_type":"markdown","metadata":{"id":"RpfWknW8CxDE"},"source":["* _**currentThread()**_: Devuelve una referencia al hilo en ejecución actual. Esto es útil cuando necesitas acceder a información o realizar acciones específicas en el hilo actual."]},{"cell_type":"code","execution_count":null,"metadata":{"id":"Z9YxyYqhCxDE","vscode":{"languageId":"java"}},"outputs":[],"source":["class MiHilo extends Thread {\n","    @Override\n","    public void run() {\n","        System.out.println(\"Ejecutando el hilo actual: \" + currentThread().getName());\n","    }\n","}\n","\n","public class Clase8 {\n","    public static void main() {\n","        MiHilo hilo1 = new MiHilo();\n","        MiHilo hilo2 = new MiHilo();\n","\n","        hilo1.setName(\"Hilo 1\");\n","        hilo2.setName(\"Hilo 2\");\n","\n","        hilo1.start(); // Inicia la ejecución del hilo1\n","        hilo2.start(); // Inicia la ejecución del hilo2\n","\n","        try {\n","            hilo1.join();\n","            hilo2.join();\n","        } catch (Exception e) {\n","            System.out.println(\"__\");\n","        }\n","    }\n","}"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"l7B3Ic1HCxDF","outputId":"0172229b-ee95-418a-e903-84c70d83785e","vscode":{"languageId":"java"}},"outputs":[{"name":"stdout","output_type":"stream","text":["Ejecutando el hilo actual: Hilo 1\n","Ejecutando el hilo actual: Hilo 2\n"]}],"source":["Clase8.main();"]},{"cell_type":"markdown","metadata":{"id":"-7mqY-3wCxDF"},"source":["* wait(): Se utiliza en la programación multihilo para coordinar la comunicación entre hilos\n","\n","Este método solo se puede llamar dentro de un bloque sincronizado, es decir, el hilo que lo llama debe tener el bloqueo del objeto en el que se invoca **wait()**.\n","\n","1. Cuando un hilo invoca **wait()**, libera el bloqueo del objeto y entra en un estado de espera. El hilo permanecerá en estado de espera hasta que otro hilo invoque el método **notify()** o **notifyAll()** en el mismo objeto.\n","\n","2. La ejecución del hilo se reanuda cuando se recibe una notificación (**notify()** o **notifyAll()**) en el objeto y el hilo puede volver a adquirir el bloqueo del objeto.\n","\n","3. Existen tres versiones del método **wait()**:\n","\n","- **wait()**: Hace que el hilo actual espere hasta que otro hilo invoque el método **notify()** o **notifyAll()** en el objeto.\n","- **wait(long timeout)**: Hace que el hilo actual espere hasta que otro hilo invoque el método **notify()** o **notifyAll()** en el objeto, o hasta que transcurra un tiempo de espera específico (en milisegundos).\n","- **wait(long timeout, int nanos)**: Hace que el hilo actual espere hasta que otro hilo invoque el método **notify()** o **notifyAll()** en el objeto, o hasta que transcurra un tiempo de espera específico (en milisegundos y nanosegundos).\n","\n","4. El método **wait()** puede lanzar una InterruptedException si el hilo que lo invoca es interrumpido mientras espera.\n","\n"]},{"cell_type":"markdown","metadata":{"id":"Ff4Uxz_7CxDG"},"source":["### Ejemplo "]},{"cell_type":"code","execution_count":null,"metadata":{"colab":{"base_uri":"https://localhost:8080/","height":130},"executionInfo":{"elapsed":6,"status":"error","timestamp":1681394463598,"user":{"displayName":"JAMES ROJAS MUÑOZ","userId":"11787007918613424953"},"user_tz":300},"id":"N8rm2BGUCxDG","outputId":"151fd3c6-582d-46e7-8f0b-dfa96520c7ef","vscode":{"languageId":"java"}},"outputs":[{"ename":"SyntaxError","evalue":"ignored","output_type":"error","traceback":["\u001b[0;36m  File \u001b[0;32m\"<ipython-input-1-dffafce0b58d>\"\u001b[0;36m, line \u001b[0;32m1\u001b[0m\n\u001b[0;31m    class Buffer {\u001b[0m\n\u001b[0m                 ^\u001b[0m\n\u001b[0;31mSyntaxError\u001b[0m\u001b[0;31m:\u001b[0m invalid syntax\n"]}],"source":["class Buffer {\n","    private Integer contenido;\n","    private boolean disponible = false;\n","\n","    public synchronized void poner(int valor) throws InterruptedException {\n","        while (disponible) {\n","           wait(); // Espera hasta que el consumidor consuma el contenido\n","        }\n","        contenido = valor;\n","        disponible = true;\n","        System.out.println(\"Producido: \" + contenido);\n","        notify(); // Notifica al consumidor que el contenido está disponible\n","    }\n","\n","    public synchronized void tomar() throws InterruptedException {\n","        while (!disponible) {\n","            wait(); // Espera hasta que el productor produzca el contenido\n","        }\n","        disponible = false;\n","        System.out.println(\"Consumido: \" + contenido);\n","        notify(); // Notifica al productor que el contenido ha sido consumido\n","    }\n","}\n","\n","class Productor implements Runnable {\n","    private Buffer buffer;\n","\n","    public Productor(Buffer buffer) {\n","        this.buffer = buffer;\n","    }\n","\n","    @Override\n","    public void run() {\n","        for (int i = 1; i < 10; i++) {\n","            try {\n","                buffer.poner(i);\n","                Thread.sleep(1000); // Simula tiempo de producción\n","            } catch (Exception e) {\n","                e.printStackTrace();\n","            }\n","        }\n","    }\n","}\n","\n","class Consumidor implements Runnable {\n","    private Buffer buffer;\n","\n","    public Consumidor(Buffer buffer) {\n","        this.buffer = buffer;\n","    }\n","\n","    @Override\n","    public void run() {\n","        for (int i = 1; i < 10; i++) {\n","            try {\n","                buffer.tomar();\n","                Thread.sleep(500); // Simula tiempo de consumo\n","            } catch (Exception e) {\n","                e.printStackTrace();\n","            }\n","        }\n","    }\n","}\n","\n","public class Clase9 {\n","    public static void main() {\n","        Buffer buffer = new Buffer();\n","         \n","        Productor productor = new Productor(buffer);\n","        Consumidor consumidor = new Consumidor(buffer);\n","        \n","        Thread hiloProductor = new Thread(productor);\n","        Thread hiloConsumidor = new Thread(consumidor);\n","\n","        hiloProductor.start();\n","        hiloConsumidor.start();\n","\n","        try {\n","            hiloProductor.join();\n","            hiloConsumidor.join();\n","        } catch (Exception e) {\n","            System.out.println(\"__\");\n","        }\n","    }\n","}"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"PO6z9mePCxDG","outputId":"9fd38b36-fec2-4564-dbce-78734aa72675","vscode":{"languageId":"java"}},"outputs":[{"name":"stdout","output_type":"stream","text":["Producido: 1\n","Consumido: 1\n","Producido: 2\n","Consumido: 2\n","Producido: 3\n","Consumido: 3\n","Producido: 4\n","Consumido: 4\n","Producido: 5\n","Consumido: 5\n","Producido: 6\n","Consumido: 6\n","Producido: 7\n","Consumido: 7\n","Producido: 8\n","Consumido: 8\n","Producido: 9\n","Consumido: 9\n"]}],"source":["Clase9.main();"]},{"cell_type":"code","execution_count":null,"metadata":{"id":"RD15wmMb4fPy","vscode":{"languageId":"java"}},"outputs":[],"source":[]},{"cell_type":"code","execution_count":null,"metadata":{"id":"7nEvrNPa4fPz","vscode":{"languageId":"java"}},"outputs":[],"source":[]}],"metadata":{"colab":{"provenance":[]},"kernelspec":{"display_name":"Java","language":"java","name":"java_tcp"},"language_info":{"codemirror_mode":"java","file_extension":".jshell","mimetype":"text/x-java-source","name":"Java","pygments_lexer":"java","version":"11.0.18+10-post-Ubuntu-0ubuntu122.04"},"orig_nbformat":4},"nbformat":4,"nbformat_minor":0}
